# update.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys, re, getopt
from command import Command
import stdhome.the as the
from stdhome.deployment import Deployment


class UpdateCommand( Command ):


	def print_help( self ):
		print "Usage: " + the.program.name + " update [--repo=REPO]"
		print
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print "Update a local repository and deploy changes."
		print
		print "Files in your home directory are copied in to the local repository, the local"
		print "repository is then updated (i.e., changes made to it are pulled down and merged)"
		print "and files from the local repository are then redeployed to your home directory."
		print
		print "If any conflicts occur, either during the merge or during the redeployment, no"
		print "files will be deployed and you will be expected to manually intervene.  Merge"
		print "conflicts should be dealt with the same way as you'd expect for version control."
		print "Conflicts that arise from files already existing in your home directory must be"
		print "dealt with by moving those files aside (currently)."
		print
		print "After a failed update, you can list outstanding conflicts by typing:"
		print "    " + the.program.name + " conflicts"
		print
		print "After fixing outstanding conflicts, you can re-attempt the redeployment of"
		print "your repository by typing:"
		print "    " + the.program.name + " resolve"
		print
		print "To back out of trying to update and revert the local repository, type:"
		print "    " + the.program.name + " stage-revert"
		print
		print "Options:"
		print "  -r, --repo=REPO  select the repo to check-out or create (defaults to 'home')"
		print "  -v, --verbose    display information about what is being done"
		print "      --help       display help and exit"
		exit( 0 )


	def parse_command_line( self ):
		opts, args = getopt.gnu_getopt(
			sys.argv[ 1: ], "r:v",
			[ "repo=", "verbose", "help" ] )
		for opt, optarg in opts:
			if opt in [ '--repo', '-r' ]:
				if not re.match( '^[-a-zA-z0-9.]+$', optarg ):
					raise the.program.FatalError(
						'invalid repository name: ' + optarg )
				the.repo = optarg
			elif opt == "--help":
				self.print_help()

		# discard first argument (the command)
		args.pop( 0 )

		# remaining arguments
		if len( args ):
			raise the.program.UsageError( 'too many arguments' )


	def run( self ):

		# set up repo and check it exists
		the.repo.check_dir_exists()

		# initialise deployment (and check it)
		deployment = Deployment()
		deployment.check_ongoing( False )

		# check for local changes
		if the.repo.vcs.has_changes():
			raise the.program.FatalError(
				'repo has local changes: %s\n'
				'Hint: see "%s stage-revert --help"' %
				( the.repo.name, the.program.name ) )

		# copy-in changes to repo
		try:
			deployment.copy_in()
		except deployment.CopyInConflicts as e:
			raise the.program.FatalError(
				'Files in %s differ too severly from %s:\n  %s\n' % \
				( the.home_dir, the.repo.dir, '\n  '.join( e.conflicts ) ) )

		# perform vcs update
		if the.verbose >= 1: print "updating %s" % the.repo.dir
		updated_files = the.repo.vcs.update()

		message = ''

		# check for conflicts in repo
		files = the.repo.vcs.get_conflicts()
		if files:
			message += 'conflicts in %s:\n  %s' % \
					   ( the.repo.name, '\n  '.join( files ) )

		# check for deployment conclicts
		conflicts = deployment.get_conflicts( updated_files )
		if conflicts:
			message += 'deployment conflicts:\n  %s' % \
					   '\n  '.join( conflicts )

		# stop if there are conflicts
		if message:
			raise the.program.FatalError(
				'there were conflicts...\n' + message )
		# copy-out changes from repo
		deployment.copy_out()

		# now we've copied-out, revert any copied-in changes!
		if the.verbose >= 1: print "reverting %s" % the.repo.dir
		the.repo.vcs.revert()
